@@ -46,11 +46,13 @@ import (
 
 // ampliconCmd represents the amplicon command
 var ampliconCmd = &cobra.Command{
+	GroupID: "search",
+
 	Use:   "amplicon",
 	Short: "extract amplicon (or specific region around it) via primer(s)",
 	Long: `extract amplicon (or specific region around it) via primer(s).
 
-Attentions:
+Attention:
   1. Only one (the longest) matching location is returned for every primer pair.
   2. Mismatch is allowed, but the mismatch location (5' or 3') is not controlled.
      You can increase the value of "-j/--threads" to accelerate processing.
@@ -191,6 +193,22 @@ Examples:
 
 		strands := []string{"+", "-"}
 
+		// -----------
+		// finder pools
+		finderPools := make([]*sync.Pool, len(primers))
+		for i, primer := range primers {
+			f := make([]byte, len(primer[1])) // copy the primer sequence
+			copy(f, primer[1])
+
+			r := make([]byte, len(primer[2])) // copy the primer sequence
+			copy(r, primer[2])
+
+			finderPools[i] = &sync.Pool{New: func() interface{} {
+				finder, _ := NewAmpliconFinder([]byte{'A'}, f, r, maxMismatch)
+				return finder
+			}}
+		}
+
 		// -------------------------------------------------------------------
 		// only for m > 0, where FMI is slow
 
@@ -276,9 +294,8 @@ Examples:
 			var id uint64
 			for _, file := range files {
 				var record *fastx.Record
-				var fastxReader *fastx.Reader
 
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 
 				for {
@@ -304,12 +321,14 @@ Examples:
 							<-tokens
 						}()
 
+						var j int
 						var finder *AmpliconFinder
 						var loc []int
 						var mis []int
 						var strand string
 						var tmpSeq *seq.Seq
 						var primer [3][]byte
+						var start1, end1 int
 
 						results := make([]string, 0, 2)
 						var s []byte
@@ -323,9 +342,11 @@ Examples:
 								record.Seq.RevComInplace()
 							}
 
-							for _, primer = range primers {
-								finder, err = NewAmpliconFinder(record.Seq.Seq, primer[1], primer[2], maxMismatch)
-								checkError(err)
+							for j, primer = range primers {
+								// finder, err = NewAmpliconFinder(record.Seq.Seq, primer[1], primer[2], maxMismatch)
+								// checkError(err)
+								finder = finderPools[j].Get().(*AmpliconFinder)
+								finder.Reset(record.Seq.Seq, maxMismatch)
 
 								if usingRegion {
 									loc, mis, err = finder.LocateRange(begin, end, fregion, strict)
@@ -339,12 +360,16 @@ Examples:
 								}
 
 								if outFmtBED {
+									start1, end1 = loc[0]-1, loc[1]
+									if strand == "-" {
+										start1, end1 = len(record.Seq.Seq)-loc[1], len(record.Seq.Seq)-loc[0]+1
+									}
 									if outputMismatches {
 										s = record.Seq.SubSeq(loc[0], loc[1]).Seq
 										results = append(results, fmt.Sprintf("%s\t%d\t%d\t%s\t%d\t%s\t%s\t%d\t%d\t%d\n",
 											record.ID,
-											loc[0]-1,
-											loc[1],
+											start1,
+											end1,
 											primer[0],
 											0,
 											strand,
@@ -356,8 +381,8 @@ Examples:
 									} else {
 										results = append(results, fmt.Sprintf("%s\t%d\t%d\t%s\t%d\t%s\t%s\n",
 											record.ID,
-											loc[0]-1,
-											loc[1],
+											start1,
+											end1,
 											primer[0],
 											0,
 											strand,
@@ -375,6 +400,8 @@ Examples:
 								results = append(results, string(record.Format(config.LineWidth)))
 
 								record.Seq = tmpSeq
+
+								finderPools[j].Put(finder)
 							}
 						}
 
@@ -394,6 +421,7 @@ Examples:
 
 					}(record.Clone(), id)
 				}
+				fastxReader.Close()
 
 				config.LineWidth = lineWidth
 			}
@@ -406,8 +434,8 @@ Examples:
 		}
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 
+		var j int
 		var finder *AmpliconFinder
 		var loc []int
 
@@ -416,9 +444,10 @@ Examples:
 		var primer [3][]byte
 		var matched bool
 		var name0 string
+		var start1, end1 int
 
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			for {
@@ -446,9 +475,11 @@ Examples:
 						record.Seq.RevComInplace()
 					}
 
-					for _, primer = range primers {
-						finder, err = NewAmpliconFinder(record.Seq.Seq, primer[1], primer[2], maxMismatch)
-						checkError(err)
+					for j, primer = range primers {
+						// finder, err = NewAmpliconFinder(record.Seq.Seq, primer[1], primer[2], maxMismatch)
+						// checkError(err)
+						finder = finderPools[j].Get().(*AmpliconFinder)
+						finder.Reset(record.Seq.Seq, maxMismatch)
 
 						if usingRegion {
 							loc, _, err = finder.LocateRange(begin, end, fregion, strict)
@@ -464,12 +495,16 @@ Examples:
 						matched = true
 
 						if outFmtBED {
+							start1, end1 = loc[0]-1, loc[1]
+							if strand == "-" {
+								start1, end1 = len(record.Seq.Seq)-loc[1], len(record.Seq.Seq)-loc[0]+1
+							}
 							if outputMismatches {
 								fmt.Fprintf(outfh,
 									"%s\t%d\t%d\t%s\t%d\t%s\t%s\t%d\t%d\t%d\n",
 									record.ID,
-									loc[0]-1,
-									loc[1],
+									start1,
+									end1,
 									primer[0],
 									0,
 									strand,
@@ -482,8 +517,8 @@ Examples:
 								fmt.Fprintf(outfh,
 									"%s\t%d\t%d\t%s\t%d\t%s\t%s\n",
 									record.ID,
-									loc[0]-1,
-									loc[1],
+									start1,
+									end1,
 									primer[0],
 									0,
 									strand,
@@ -501,6 +536,8 @@ Examples:
 						record.FormatToWriter(outfh, config.LineWidth)
 
 						record.Seq = tmpSeq
+
+						finderPools[j].Put(finder)
 					}
 				}
 
@@ -511,6 +548,7 @@ Examples:
 					record.FormatToWriter(outfh, config.LineWidth)
 				}
 			}
+			fastxReader.Close()
 
 			config.LineWidth = lineWidth
 		}
@@ -620,6 +658,25 @@ type AmpliconFinder struct {
 	rF, rR *regexp.Regexp
 }
 
+func (finder *AmpliconFinder) Reset(sequence []byte, maxMismatch int) error {
+	if len(sequence) == 0 {
+		return fmt.Errorf("non-blank sequence needed")
+	}
+	finder.Seq = bytes.ToUpper(sequence)
+	if maxMismatch > 0 { // using FM-index
+		index := fmi.NewFMIndex()
+		_, err := index.Transform(finder.Seq)
+		if err != nil {
+			return err
+		}
+		finder.MaxMismatch = maxMismatch
+		finder.FMindex = index
+	}
+
+	finder.searched, finder.found = false, false
+	return nil
+}
+
 // NewAmpliconFinder returns a AmpliconFinder struct.
 func NewAmpliconFinder(sequence, forwardPrimer, reversePrimerRC []byte, maxMismatch int) (*AmpliconFinder, error) {
 	if len(sequence) == 0 {
@@ -713,21 +770,20 @@ func (finder *AmpliconFinder) LocateRange(begin, end int, flanking bool, strictM
 // end: relative location to 3' end of amplicon.
 // Returned locations are 1-based.
 //
-//                     F
-//         -----===============-----
-//              1 3 5                    x/y
-//                       -5-3-1          x/y
-//              F             R
-//         -----=====-----=====-----     x:y
-//
-//              ===============          1:-1
-//              =======                  1:7
-//                =====                  3:7
-//                   =====               6:10
-//                   =====             -10:-6
-//                      =====           -7:-3
-//                                      -x:y (invalid)
+//	            F
+//	-----===============-----
+//	     1 3 5                    x/y
+//	              -5-3-1          x/y
+//	     F             R
+//	-----=====-----=====-----     x:y
 //
+//	     ===============          1:-1
+//	     =======                  1:7
+//	       =====                  3:7
+//	          =====               6:10
+//	          =====             -10:-6
+//	             =====           -7:-3
+//	                             -x:y (invalid)
 func SubLocationInner(length, B, E, begin, end int, strictMode bool) (int, int, bool) {
 	if begin == 0 || end == 0 {
 		return 0, 0, false
@@ -790,20 +846,19 @@ func SubLocationInner(length, B, E, begin, end int, strictMode bool) (int, int,
 // end: relative location to 3' end of amplicon.
 // Returned locations are 1-based.
 //
-//                     F
-//         -----===============-----
-//          -3-1                        x/y
-//                             1 3 5    x/y
-//              F             R
-//         -----=====-----=====-----
-//         =====                        -5:-1
-//         ===                          -5:-3
-//                             =====     1:5
-//                               ===     3:5
-//             =================        -1:1
-//         =========================    -5:5
-//                                       x:-y (invalid)
-//
+//	            F
+//	-----===============-----
+//	 -3-1                        x/y
+//	                    1 3 5    x/y
+//	     F             R
+//	-----=====-----=====-----
+//	=====                        -5:-1
+//	===                          -5:-3
+//	                    =====     1:5
+//	                      ===     3:5
+//	    =================        -1:1
+//	=========================    -5:5
+//	                              x:-y (invalid)
 func SubLocationFlanking(length, B, E, begin, end int, strictMode bool) (int, int, bool) {
 	if begin == 0 || end == 0 {
 		return 0, 0, false
