@@ -35,7 +35,7 @@ import (
 
 	"github.com/biogo/hts/bam"
 	"github.com/biogo/hts/sam"
-	"github.com/bsipos/thist"
+	"github.com/botond-sipos/thist"
 	"github.com/shenwei356/bio/seq"
 	"github.com/shenwei356/xopen"
 	"github.com/spf13/cobra"
@@ -489,6 +489,8 @@ type TopBuffer []topEntry
 
 // bamCmd represents the bam command
 var bamCmd = &cobra.Command{
+	GroupID: "bam",
+
 	Use:   "bam",
 	Short: "monitoring and online histograms of BAM record features",
 	Long:  "monitoring and online histograms of BAM record features",
