@@ -23,6 +23,7 @@ package cmd
 import (
 	"bytes"
 	"fmt"
+	"os"
 	"regexp"
 	"runtime"
 	"strconv"
@@ -38,9 +39,11 @@ import (
 
 // faidxCmd represents the faidx command
 var faidxCmd = &cobra.Command{
+	GroupID: "basic",
+
 	Use:   "faidx",
-	Short: "create FASTA index file and extract subsequence",
-	Long: fmt.Sprintf(`create FASTA index file and extract subsequence
+	Short: "create the FASTA index file and extract subsequences",
+	Long: fmt.Sprintf(`create the FASTA index file and extract subsequences
 
 This command is similar with "samtools faidx" but has some extra features:
 
@@ -49,6 +52,9 @@ This command is similar with "samtools faidx" but has some extra features:
   3. if you have large number of IDs, you can use:
         seqkit faidx seqs.fasta -l IDs.txt
 
+Attention:
+  1. The flag -U/--update-faidx is recommended to ensure the .fai file matches the FASTA file.
+
 The definition of region is 1-based and with some custom design.
 
 Examples:
@@ -67,6 +73,8 @@ Examples:
 
 		immediateOutput := getFlagBool(cmd, "immediate-output")
 
+		updateFaidx := getFlagBool(cmd, "update-faidx")
+
 		files := getFileListFromArgsAndFile(cmd, args, false, "infile-list", false)
 
 		file := files[0]
@@ -125,8 +133,20 @@ Examples:
 			idRegexp = "^(.+)$"
 		} else {
 			fileFai = file + ".fai"
-			idRegexp = fastx.DefaultIDRegexp
+			idRegexp = config.IDRegexp
 		}
+
+		if !quiet {
+			log.Infof("create or read FASTA index ...")
+		}
+
+		if FileExists(fileFai) && updateFaidx {
+			checkError(os.RemoveAll(fileFai))
+			if !quiet {
+				log.Infof("delete the old FASTA index file: %s", fileFai)
+			}
+		}
+
 		if fileNotExists(fileFai) {
 			if !quiet {
 				log.Infof("create FASTA index for %s", file)
@@ -134,8 +154,18 @@ Examples:
 			idx, err = fai.CreateWithIDRegexp(file, fileFai, idRegexp)
 			checkError(err)
 		} else {
+			if !quiet {
+				log.Infof("read FASTA index from %s", fileFai)
+			}
 			idx, err = fai.Read(fileFai)
 			checkError(err)
+
+			if len(idx) == 0 {
+				log.Warningf("0 records loaded from %s, please check if it matches the FASTA file, or switch on the flag -U/--update-faidx", fileFai)
+				return
+			} else if !quiet {
+				log.Infof("  %d records loaded from %s", len(idx), fileFai)
+			}
 		}
 
 		if len(files) == 1 { // just creat .fai file
@@ -153,7 +183,7 @@ Examples:
 		id2head := make(map[string]string)
 		var idRe *regexp.Regexp
 		if fullHead {
-			idRe, _ = regexp.Compile(fastx.DefaultIDRegexp)
+			idRe, _ = regexp.Compile(config.IDRegexp)
 		}
 		var id string
 		for head := range idx {
@@ -276,6 +306,7 @@ func init() {
 	faidxCmd.Flags().StringP("region-file", "l", "", "file containing a list of regions")
 
 	faidxCmd.Flags().BoolP("immediate-output", "I", false, "print output immediately, do not use write buffer")
+	faidxCmd.Flags().BoolP("update-faidx", "U", false, "update the fasta index file if it exists. Use this if you are not sure whether the fasta file changed")
 
 	faidxCmd.SetUsageTemplate(`Usage:{{if .Runnable}}
   {{if .HasAvailableFlags}}{{appendIfNotPresent .UseLine "[flags]"}}{{else}}{{.UseLine}}{{end}}{{end}}{{if .HasAvailableSubCommands}}
@@ -291,10 +322,10 @@ Available Commands:{{range .Commands}}{{if .IsAvailableCommand}}
   {{rpad .Name .NamePadding }} {{.Short}}{{end}}{{end}}{{end}}{{ if .HasAvailableLocalFlags}}
 
 Flags:
-{{.LocalFlags.FlagUsages | trimRightSpace}}{{end}}{{ if .HasAvailableInheritedFlags}}
+{{.LocalFlags.FlagUsagesWrapped 110 | trimTrailingWhitespaces}}{{end}}{{if .HasAvailableInheritedFlags}}
 
 Global Flags:
-{{.InheritedFlags.FlagUsages | trimRightSpace}}{{end}}{{if .HasHelpSubCommands}}
+{{.InheritedFlags.FlagUsagesWrapped 110 | trimTrailingWhitespaces}}{{end}}{{if .HasHelpSubCommands}}
 
 Additional help topics:{{range .Commands}}{{if .IsHelpCommand}}
   {{rpad .CommandPath .CommandPathPadding}} {{.Short}}{{end}}{{end}}{{end}}{{ if .HasAvailableSubCommands }}
