@@ -111,6 +111,8 @@ func parseRanges(rf string) Ranges {
 
 // fishCmd represents the fish command
 var fishCmd = &cobra.Command{
+	GroupID: "search",
+
 	Use:   "fish",
 	Short: "look for short sequences in larger sequences using local alignment",
 	Long: `look for short sequences in larger sequences using local alignment
@@ -152,11 +154,8 @@ Attention:
 		alnParams := parseAlnParams(flagAlnParams)
 
 		validateSeq := getFlagBool(cmd, "validate-seq")
-		validateSeqLength := getFlagValidateSeqLength(cmd, "validate-seq-length")
 
 		seq.ValidateSeq = validateSeq
-		seq.ValidateWholeSeq = false
-		seq.ValidSeqLengthThreshold = validateSeqLength
 		seq.ValidSeqThreads = config.Threads
 
 		if !(alphabet == nil || alphabet == seq.Unlimit) {
@@ -189,7 +188,6 @@ Attention:
 		var buffer *bytes.Buffer
 		var record *fastx.Record
 		var sequence *seq.Seq
-		var fastxReader *fastx.Reader
 		var count int
 		var refMap map[string]int
 		var samRefs []*sam.Reference
@@ -201,7 +199,7 @@ Attention:
 		first := true
 
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			checkSeqType = true
@@ -292,8 +290,8 @@ Attention:
 
 					outfh.Write(_mark_newline)
 				}
-
 			} // record
+			fastxReader.Close()
 			config.LineWidth = lineWidth
 
 		} //file
@@ -359,7 +357,6 @@ func init() {
 	fishCmd.Flags().BoolP("print-desc", "D", false, "print full sequence header")
 	fishCmd.Flags().BoolP("print-aln", "g", false, "print sequence alignments")
 	fishCmd.Flags().BoolP("invert", "i", false, "print out references not matching with any query")
-	fishCmd.Flags().IntP("validate-seq-length", "V", 10000, "length of sequence to validate (0 for whole seq)")
 	fishCmd.Flags().Float64P("min-qual", "q", 5.0, "minimum mapping quality")
 }
 
