@@ -32,6 +32,8 @@ import (
 
 // fq2faCmd represents the fq2fa command
 var fq2faCmd = &cobra.Command{
+	GroupID: "format",
+
 	Use:   "fq2fa",
 	Short: "convert FASTQ to FASTA",
 	Long: `convert FASTQ to FASTA
@@ -54,9 +56,8 @@ var fq2faCmd = &cobra.Command{
 		defer outfh.Close()
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			for {
@@ -73,6 +74,7 @@ var fq2faCmd = &cobra.Command{
 				// record.FormatToWriter(outfh, lineWidth)
 				record.FormatToWriter(outfh, 0)
 			}
+			fastxReader.Close()
 		}
 	},
 }
