@@ -30,6 +30,7 @@ import (
 	"sort"
 	"strings"
 
+	"github.com/elliotwutingfeng/asciiset"
 	"github.com/shenwei356/bio/seq"
 	"github.com/shenwei356/bio/seqio/fastx"
 	"github.com/shenwei356/xopen"
@@ -38,6 +39,8 @@ import (
 
 // fx2tabCmd represents the fx2tab command
 var fx2tabCmd = &cobra.Command{
+	GroupID: "format",
+
 	Use:   "fx2tab",
 	Short: "convert FASTA/Q to tabular format (and length, GC content, average quality...)",
 	Long: `convert FASTA/Q to tabular format, and provide various information,
@@ -136,10 +139,9 @@ Attention:
 		var name []byte
 		var g, c float64
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		var sum [md5.Size]byte
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 			for {
 				record, err = fastxReader.Read()
@@ -228,6 +230,7 @@ Attention:
 				// outfh.WriteString("\n")
 				outfh.Write(_mark_newline)
 			}
+			fastxReader.Close()
 		}
 	},
 }
@@ -253,16 +256,14 @@ func init() {
 }
 
 func alphabetStr(s []byte) string {
-	m := make(map[byte]struct{})
-	for _, b := range s {
-		m[b] = struct{}{}
-	}
-	alphabet := make([]string, len(m))
+	m, _ := asciiset.MakeASCIISet(string(s))
+	alphabet := make([]string, m.Size())
 	i := 0
-	for a := range m {
+	m.Visit(func(a byte) bool {
 		alphabet[i] = string([]byte{a})
 		i++
-	}
+		return false
+	})
 	sort.Strings(alphabet)
 	return strings.Join(alphabet, "")
 }
