@@ -32,12 +32,14 @@ import (
 
 // headCmd represents the head command
 var headCmd = &cobra.Command{
+	GroupID: "set",
+
 	Use:   "head",
 	Short: "print first N FASTA/Q records",
 	Long: `print first N FASTA/Q records
 
 For returning the last N records, use:
-    seqkit range -N:-1 seqs.fasta
+    seqkit range -r -N:-1 seqs.fasta
 
 `,
 	Run: func(cmd *cobra.Command, args []string) {
@@ -59,10 +61,9 @@ For returning the last N records, use:
 		defer outfh.Close()
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		i := 0
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			for {
@@ -85,6 +86,7 @@ For returning the last N records, use:
 					return
 				}
 			}
+			fastxReader.Close()
 
 			config.LineWidth = lineWidth
 		}
