@@ -37,11 +37,13 @@ import (
 
 // mutateCmd represents the mutate command
 var mutateCmd = &cobra.Command{
+	GroupID: "edit",
+
 	Use:   "mutate",
 	Short: "edit sequence (point mutation, insertion, deletion)",
 	Long: fmt.Sprintf(`edit sequence (point mutation, insertion, deletion)
 
-Attentions:
+Attention:
 
   1. Multiple point mutations (-p/--point) are allowed, but only single
      insertion (-i/--insertion) OR single deletion (-d/--deletion) is allowed.
@@ -204,7 +206,6 @@ Examples:
 		defer outfh.Close()
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		var checkFQ = true
 		var mp _mutatePoint
 		var seqLen int
@@ -215,7 +216,7 @@ Examples:
 		var k string
 		var re *regexp.Regexp
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			for {
@@ -326,6 +327,7 @@ Examples:
 				}
 				record.FormatToWriter(outfh, lineWidth)
 			}
+			fastxReader.Close()
 		}
 	},
 }
