@@ -35,6 +35,8 @@ import (
 
 // rangeCmd represents the range command
 var rangeCmd = &cobra.Command{
+	GroupID: "set",
+
 	Use:   "range",
 	Short: "print FASTA/Q records in a range (start:end)",
 	Long: `print FASTA/Q records in a range (start:end)
@@ -115,9 +117,8 @@ Examples:
 		}
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		for _, file := range files {
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
 			if start < 0 && end < 0 {
@@ -166,6 +167,7 @@ Examples:
 					buf.Add(record.Clone())
 				}
 			}
+			fastxReader.Close()
 
 			if rangeNN {
 				current0 := buf.Current
