@@ -37,6 +37,8 @@ import (
 
 // renameCmd represents the rename command
 var renameCmd = &cobra.Command{
+	GroupID: "edit",
+
 	Use:   "rename",
 	Short: "rename duplicated IDs",
 	Long: `rename duplicated IDs
@@ -73,12 +75,14 @@ Example:
 		files := getFileListFromArgsAndFile(cmd, args, true, "infile-list", true)
 
 		byName := getFlagBool(cmd, "by-name")
-		// inPlace := getFlagBool(cmd, "inplace")
-		inPlace := true
 		mOutputs := getFlagBool(cmd, "multiple-outfiles")
 		outdir := getFlagString(cmd, "out-dir")
 		force := getFlagBool(cmd, "force")
 
+		separator := getFlagString(cmd, "separator")
+		startNum := getFlagPositiveInt(cmd, "start-num")
+		rename1st := getFlagBool(cmd, "rename-1st-rec")
+
 		var outfh *xopen.Writer
 		var err error
 
@@ -118,14 +122,13 @@ Example:
 		}
 
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		var newID string
 		var k uint64
 		var ok bool
 		numbers := make(map[uint64]int, 1<<20)
 		for _, file := range files {
 			func(file string) {
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 
 				if mOutputs {
@@ -156,18 +159,27 @@ Example:
 
 					if _, ok = numbers[k]; ok {
 						numbers[k]++
-						newID = fmt.Sprintf("%s_%d", record.ID, numbers[k])
-						if inPlace {
+						newID = fmt.Sprintf("%s%s%d", record.ID, separator, numbers[k]-2+startNum)
+						if len(record.Desc) > 0 {
 							record.Name = []byte(fmt.Sprintf("%s %s", newID, record.Desc))
 						} else {
-							record.Name = []byte(fmt.Sprintf("%s %s", newID, record.Name))
+							record.Name = []byte(newID)
 						}
 					} else {
 						numbers[k] = 1
+						if rename1st {
+							newID = fmt.Sprintf("%s%s%d", record.ID, separator, numbers[k]-2+startNum)
+							if len(record.Desc) > 0 {
+								record.Name = []byte(fmt.Sprintf("%s %s", newID, record.Desc))
+							} else {
+								record.Name = []byte(newID)
+							}
+						}
 					}
 
 					record.FormatToWriter(outfh, config.LineWidth)
 				}
+				fastxReader.Close()
 				config.LineWidth = lineWidth
 			}(file)
 		}
@@ -177,9 +189,12 @@ Example:
 func init() {
 	RootCmd.AddCommand(renameCmd)
 
+	renameCmd.Flags().StringP("separator", "s", "_", "separator between original ID/name and the counter")
+	renameCmd.Flags().IntP("start-num", "N", 2, `starting count number for *duplicated* IDs/names, should be greater than zero`)
+	renameCmd.Flags().BoolP("rename-1st-rec", "1", false, "rename the first record as well")
+
 	renameCmd.Flags().BoolP("by-name", "n", false, "check duplication by full name instead of just id")
 	renameCmd.Flags().BoolP("multiple-outfiles", "m", false, "write results into separated files for multiple input files")
 	renameCmd.Flags().StringP("out-dir", "O", "renamed", "output directory")
 	renameCmd.Flags().BoolP("force", "f", false, "overwrite output directory")
-	// renameCmd.Flags().BoolP("inplace", "i", false, "rename ID in-place")
 }
