@@ -34,6 +34,8 @@ import (
 
 // sampleCmd represents the sample command
 var sampleCmd = &cobra.Command{
+	GroupID: "set",
+
 	Use:   "sample",
 	Short: "sample sequences by number or proportion",
 	Long: `sample sequences by number or proportion.
@@ -91,7 +93,6 @@ Attention:
 
 		n := int64(0)
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		if number > 0 { // by number
 			if !quiet {
 				log.Info("sample by number")
@@ -115,7 +116,7 @@ Attention:
 				if !quiet {
 					log.Info("second pass: reading and sampling")
 				}
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 			LOOP:
 				for {
@@ -141,6 +142,7 @@ Attention:
 						}
 					}
 				}
+				fastxReader.Close()
 			} else {
 				if !quiet {
 					log.Info("loading all sequences into memory...")
@@ -170,7 +172,7 @@ Attention:
 				log.Info("sample by proportion")
 			}
 
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 			for {
 				record, err = fastxReader.Read()
@@ -192,6 +194,7 @@ Attention:
 					record.FormatToWriter(outfh, config.LineWidth)
 				}
 			}
+			fastxReader.Close()
 		}
 
 		if !quiet {
@@ -203,7 +206,7 @@ Attention:
 func init() {
 	RootCmd.AddCommand(sampleCmd)
 
-	sampleCmd.Flags().Int64P("rand-seed", "s", 11, "rand seed")
+	sampleCmd.Flags().Int64P("rand-seed", "s", 11, "random seed. For paired-end data, use the same seed across fastq files to sample the same read pairs")
 	sampleCmd.Flags().Int64P("number", "n", 0, "sample by number (result may not exactly match), DO NOT use on large FASTQ files.")
 	sampleCmd.Flags().Float64P("proportion", "p", 0, "sample by proportion")
 	sampleCmd.Flags().BoolP("two-pass", "2", false, "2-pass mode read files twice to lower memory usage. Not allowed when reading from stdin")
