@@ -37,19 +37,27 @@ import (
 
 // shuffleCmd represents the shuffle command
 var shuffleCmd = &cobra.Command{
+	GroupID: "order",
+
 	Use:   "shuffle",
 	Short: "shuffle sequences",
 	Long: `shuffle sequences.
 
 By default, all records will be readed into memory.
 For FASTA format, use flag -2 (--two-pass) to reduce memory usage. FASTQ not
-supported.
+supported. 
 
 Firstly, seqkit reads the sequence IDs. If the file is not plain FASTA file,
 seqkit will write the sequences to temporary files, and create FASTA index.
 
 Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 
+Attention:
+  1. For the two-pass mode (-2/--two-pass), The flag -U/--update-faidx is recommended to
+     ensure the .fai file matches the FASTA file.
+
+
+
 `,
 	Run: func(cmd *cobra.Command, args []string) {
 		config := getConfigs(cmd)
@@ -67,14 +75,17 @@ Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 
 		seed := getFlagInt64(cmd, "rand-seed")
 		twoPass := getFlagBool(cmd, "two-pass")
+		updateFaidx := getFlagBool(cmd, "update-faidx")
 		keepTemp := getFlagBool(cmd, "keep-temp")
 		if keepTemp && !twoPass {
 			checkError(fmt.Errorf("flag -k (--keep-temp) must be used with flag -2 (--two-pass)"))
 		}
+		if updateFaidx && !twoPass {
+			checkError(fmt.Errorf("flag -U (--update-faidx) must be used with flag -2 (--two-pass)"))
+		}
 
 		index2name := make(map[int]string)
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		var err error
 
 		if !twoPass { // read all records into memory
@@ -85,7 +96,7 @@ Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 			}
 			i := 0
 			for _, file := range files {
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 				for {
 					record, err = fastxReader.Read()
@@ -105,6 +116,7 @@ Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 					index2name[i] = string(record.Name)
 					i++
 				}
+				fastxReader.Close()
 			}
 
 			if !quiet {
@@ -169,17 +181,30 @@ Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 			}
 		}
 
-		if !quiet {
-			log.Infof("create and read FASTA index ...")
+		fileFai := newFile + ".seqkit.fai"
+
+		if FileExists(fileFai) && updateFaidx {
+			checkError(os.RemoveAll(fileFai))
+			if !quiet {
+				log.Infof("delete the old FASTA index file: %s", fileFai)
+			}
 		}
-		faidx := getFaidx(newFile, `^(.+)$`)
+
+		if !quiet {
+			log.Infof("create or read FASTA index ...")
+		}
+		faidx := getFaidx(newFile, `^(.+)$`, quiet)
 		defer func() {
 			checkError(faidx.Close())
 		}()
 
-		if !quiet {
-			log.Infof("read sequence IDs from FASTA index ...")
+		if len(faidx.Index) == 0 {
+			log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+			return
+		} else if !quiet {
+			log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
 		}
+
 		ids, _, err := getSeqIDAndLengthFromFaidxFile(newFile + ".seqkit.fai")
 		checkError(err)
 		for i, id := range ids {
@@ -187,7 +212,6 @@ Secondly, seqkit shuffles sequence IDs and extract sequences by FASTA index.
 		}
 
 		if !quiet {
-			log.Infof("%d sequences loaded", len(ids))
 			log.Infof("shuffle ...")
 		}
 		rand.Seed(seed)
@@ -234,4 +258,5 @@ func init() {
 	shuffleCmd.Flags().Int64P("rand-seed", "s", 23, "rand seed for shuffle")
 	shuffleCmd.Flags().BoolP("two-pass", "2", false, "two-pass mode read files twice to lower memory usage. (only for FASTA format)")
 	shuffleCmd.Flags().BoolP("keep-temp", "k", false, "keep temporary FASTA and .fai file when using 2-pass mode")
+	shuffleCmd.Flags().BoolP("update-faidx", "U", false, "update the fasta index file if it exists. Use this if you are not sure whether the fasta file changed")
 }
