@@ -40,6 +40,8 @@ import (
 
 // sortCmd represents the sort command
 var sortCmd = &cobra.Command{
+	GroupID: "order",
+
 	Use:   "sort",
 	Short: "sort sequences by id/name/sequence/length",
 	Long: `sort sequences by id/name/sequence/length.
@@ -55,6 +57,10 @@ seqkit will write the sequences to temporary files, and create FASTA index.
 Secondly, seqkit sorts sequence by head and length information
 and extracts sequences by FASTA index.
 
+Attention:
+  1. For the two-pass mode (-2/--two-pass), The flag -U/--update-faidx is recommended to
+     ensure the .fai file matches the FASTA file.
+
 `,
 	Run: func(cmd *cobra.Command, args []string) {
 		config := getConfigs(cmd)
@@ -79,11 +85,16 @@ and extracts sequences by FASTA index.
 		reverse := getFlagBool(cmd, "reverse")
 		ignoreCase := getFlagBool(cmd, "ignore-case")
 		twoPass := getFlagBool(cmd, "two-pass")
+		updateFaidx := getFlagBool(cmd, "update-faidx")
 		seqPrefixLength := getFlagNonNegativeInt(cmd, "seq-prefix-length")
 		keepTemp := getFlagBool(cmd, "keep-temp")
 		if keepTemp && !twoPass {
 			checkError(fmt.Errorf("flag -k (--keep-temp) must be used with flag -2 (--two-pass)"))
 		}
+		if updateFaidx && !twoPass {
+			checkError(fmt.Errorf("flag -U (--update-faidx) must be used with flag -2 (--two-pass)"))
+		}
+
 		if byBases {
 			byLength = true
 
@@ -125,7 +136,6 @@ and extracts sequences by FASTA index.
 		// for indexing when output and duplicated sequences checking
 		id2name := make(map[string][]byte)
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 		var err error
 
 		if !twoPass { // read all records into memory
@@ -137,7 +147,7 @@ and extracts sequences by FASTA index.
 			var name string
 			var length int
 			for _, file := range files {
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 				for {
 					record, err = fastxReader.Read()
@@ -188,6 +198,7 @@ and extracts sequences by FASTA index.
 						}
 					}
 				}
+				fastxReader.Close()
 			}
 
 			if !quiet {
@@ -275,15 +286,31 @@ and extracts sequences by FASTA index.
 			}
 		}
 
-		if !quiet {
-			log.Infof("create and read FASTA index ...")
+		fileFai := newFile + ".seqkit.fai"
+
+		if FileExists(fileFai) && updateFaidx {
+			checkError(os.RemoveAll(fileFai))
+			if !quiet {
+				log.Infof("delete the old FASTA index file: %s", fileFai)
+			}
 		}
 
-		faidx := getFaidx(newFile, `^(.+)$`)
+		if !quiet {
+			log.Infof("create or read FASTA index ...")
+		}
+
+		faidx := getFaidx(newFile, `^(.+)$`, quiet)
 		defer func() {
 			checkError(faidx.Close())
 		}()
 
+		if len(faidx.Index) == 0 {
+			log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+			return
+		} else if !quiet {
+			log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
+		}
+
 		if !bySeq { // if not by seq, just read faidx
 			if !quiet {
 				log.Infof("read sequence IDs and lengths from FASTA index ...")
@@ -296,6 +323,7 @@ and extracts sequences by FASTA index.
 
 			ids, lengths, err := getSeqIDAndLengthFromFaidxFile(newFile + ".seqkit.fai")
 			checkError(err)
+
 			var name string
 			for i, head := range ids {
 				if byName {
@@ -373,6 +401,7 @@ and extracts sequences by FASTA index.
 				name2length = append(name2length,
 					stringutil.StringCount{Key: name, Count: len(record.Seq.Seq)})
 			}
+			fastxReader.Close()
 		}
 
 		if !quiet {
@@ -463,6 +492,7 @@ func init() {
 	sortCmd.Flags().BoolP("ignore-case", "i", false, "ignore case")
 
 	sortCmd.Flags().BoolP("two-pass", "2", false, "two-pass mode read files twice to lower memory usage. (only for FASTA format)")
+	sortCmd.Flags().BoolP("update-faidx", "U", false, "update the fasta index file if it exists. Use this if you are not sure whether the fasta file changed")
 	sortCmd.Flags().BoolP("keep-temp", "k", false, "keep temporary FASTA and .fai file when using 2-pass mode")
 	sortCmd.Flags().IntP("seq-prefix-length", "L", 10000, "length of sequence prefix on which seqkit sorts by sequences (0 for whole sequence)")
 }
