@@ -41,13 +41,31 @@ import (
 
 // splitCmd represents the split command
 var splitCmd = &cobra.Command{
+	GroupID: "set",
+
 	Use:   "split",
 	Short: "split sequences into files by id/seq region/size/parts (mainly for FASTA)",
 	Long: fmt.Sprintf(`split sequences into files by name ID, subsequence of given region,
 part size or number of parts.
 
 If you just want to split by parts or sizes, please use "seqkit split2",
-which also applies for paired- and single-end FASTQ.
+which can apply to paired- and single-end FASTQ.
+
+If you want to cut a sequence into multiple segments.
+  1. For cutting into even chunks, please use 'kmcp utils split-genomes'
+     (https://bioinf.shenwei.me/kmcp/usage/#split-genomes).
+     E.g., cutting into 4 segments of equal size, with no overlap between adjacent segments:
+        kmcp utils split-genomes -m 1 -k 1 --split-number 4 --split-overlap 0 input.fasta -O out
+  2. For cutting into multiple chunks of fixed size, please using 'seqkit sliding'.
+     E.g., cutting into segments of 40 bp and keeping the last segment which can be shorter than 40 bp.
+        seqkit sliding -g -s 40 -W 40 input.fasta -o out.fasta
+
+Attention:
+  1. For the two-pass mode (-2/--two-pass), The flag -U/--update-faidx is recommended to
+     ensure the .fai file matches the FASTA file.
+  2. For splitting by sequence IDs in Windows/MacOS, where the file systems might be case-insensitive,
+     output files might be overwritten if they are only different in cases, like Abc and ABC.
+     To avoid this, please switch one -I/--ignore-case.
 
 The definition of region is 1-based and with some custom design.
 
@@ -79,12 +97,18 @@ Examples:
 		part := getFlagNonNegativeInt(cmd, "by-part")
 
 		byID := getFlagBool(cmd, "by-id")
+		ignoreCase := getFlagBool(cmd, "ignore-case")
 		region := getFlagString(cmd, "by-region")
 		twoPass := getFlagBool(cmd, "two-pass")
+		updateFaidx := getFlagBool(cmd, "update-faidx")
 		keepTemp := getFlagBool(cmd, "keep-temp")
 		if keepTemp && !twoPass {
 			checkError(fmt.Errorf("flag -k (--keep-temp) must be used with flag -2 (--two-pass)"))
 		}
+		if updateFaidx && !twoPass {
+			checkError(fmt.Errorf("flag -U (--update-faidx) must be used with flag -2 (--two-pass)"))
+		}
+
 		dryRun := getFlagBool(cmd, "dry-run")
 
 		outdir := getFlagString(cmd, "out-dir")
@@ -128,7 +152,6 @@ Examples:
 		var outfile string
 		var prefix string
 		var record *fastx.Record
-		var fastxReader *fastx.Reader
 
 		if !dryRun {
 			pwd, _ := os.Getwd()
@@ -164,7 +187,7 @@ Examples:
 				i := 1
 				records := []*fastx.Record{}
 
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 				for {
 					record, err = fastxReader.Read()
@@ -201,6 +224,7 @@ Examples:
 						records = []*fastx.Record{}
 					}
 				}
+				fastxReader.Close()
 				if len(records) > 0 {
 					// outfile = filepath.Join(outdir, fmt.Sprintf("%s.part_%03d%s", filepath.Base(fileName), i, fileExt))
 					if prefixBySizeSet {
@@ -253,24 +277,34 @@ Examples:
 					checkError(fmt.Errorf("Sorry, two-pass mode does not support FASTQ format"))
 				}
 			}
-			fileExt = suffixFA
+			fileExt = suffixFA + extension
+
+			fileFai := newFile + ".seqkit.fai"
+
+			if FileExists(fileFai) && updateFaidx {
+				checkError(os.RemoveAll(fileFai))
+				if !quiet {
+					log.Infof("delete the old FASTA index file: %s", fileFai)
+				}
+			}
 
 			if !quiet {
-				log.Infof("create and read FASTA index ...")
+				log.Infof("create or read FASTA index ...")
 			}
-			faidx := getFaidx(newFile, `^(.+)$`)
+			faidx := getFaidx(newFile, `^(.+)$`, quiet)
 			defer func() {
 				checkError(faidx.Close())
 			}()
 
-			if !quiet {
-				log.Infof("read sequence IDs from FASTA index ...")
+			if len(faidx.Index) == 0 {
+				log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+				return
+			} else if !quiet {
+				log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
 			}
-			IDs, _, err := getSeqIDAndLengthFromFaidxFile(newFile + ".seqkit.fai")
+
+			IDs, _, err := getSeqIDAndLengthFromFaidxFile(fileFai)
 			checkError(err)
-			if !quiet {
-				log.Infof("%d sequences loaded", len(IDs))
-			}
 
 			n := 1
 			if len(IDs) > 0 {
@@ -446,24 +480,34 @@ Examples:
 					checkError(fmt.Errorf("Sorry, two-pass mode does not support FASTQ format"))
 				}
 			}
-			fileExt = suffixFA
+			fileExt = suffixFA + extension
+
+			fileFai := newFile + ".seqkit.fai"
+
+			if FileExists(fileFai) && updateFaidx {
+				checkError(os.RemoveAll(fileFai))
+				if !quiet {
+					log.Infof("delete the old FASTA index file: %s", fileFai)
+				}
+			}
 
 			if !quiet {
-				log.Infof("create and read FASTA index ...")
+				log.Infof("create or read FASTA index ...")
 			}
-			faidx := getFaidx(newFile, `^(.+)$`)
+			faidx := getFaidx(newFile, `^(.+)$`, quiet)
 			defer func() {
 				checkError(faidx.Close())
 			}()
 
-			if !quiet {
-				log.Infof("read sequence IDs from FASTA index ...")
+			if len(faidx.Index) == 0 {
+				log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+				return
+			} else if !quiet {
+				log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
 			}
-			IDs, _, err := getSeqIDAndLengthFromFaidxFile(newFile + ".seqkit.fai")
+
+			IDs, _, err := getSeqIDAndLengthFromFaidxFile(fileFai)
 			checkError(err)
-			if !quiet {
-				log.Infof("%d sequences loaded", len(IDs))
-			}
 
 			N := len(IDs)
 			if N%part > 0 {
@@ -579,6 +623,9 @@ Examples:
 						renameFileExt = false
 					}
 					id = string(record.ID)
+					if ignoreCase {
+						id = strings.ToLower(id)
+					}
 					if _, ok := recordsByID[id]; !ok {
 						recordsByID[id] = []*fastx.Record{}
 					}
@@ -641,24 +688,34 @@ Examples:
 					checkError(fmt.Errorf("sorry, two-pass mode does not support FASTQ format"))
 				}
 			}
-			fileExt = suffixFA
+			fileExt = suffixFA + extension
+
+			fileFai := newFile + ".seqkit.fai"
+
+			if FileExists(fileFai) && updateFaidx {
+				checkError(os.RemoveAll(fileFai))
+				if !quiet {
+					log.Infof("delete the old FASTA index file: %s", fileFai)
+				}
+			}
 
 			if !quiet {
-				log.Infof("create and read FASTA index ...")
+				log.Infof("create or read FASTA index ...")
 			}
-			faidx := getFaidx(newFile, `^(.+)$`)
+			faidx := getFaidx(newFile, `^(.+)$`, quiet)
 			defer func() {
 				checkError(faidx.Close())
 			}()
 
-			if !quiet {
-				log.Infof("read sequence IDs from FASTA index ...")
+			if len(faidx.Index) == 0 {
+				log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+				return
+			} else if !quiet {
+				log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
 			}
-			IDs, _, err := getSeqIDAndLengthFromFaidxFile(newFile + ".seqkit.fai")
+
+			IDs, _, err := getSeqIDAndLengthFromFaidxFile(fileFai)
 			checkError(err)
-			if !quiet {
-				log.Infof("%d sequences loaded", len(IDs))
-			}
 
 			idRe, err := regexp.Compile(idRegexp)
 			if err != nil {
@@ -668,6 +725,9 @@ Examples:
 			idsMap := make(map[string][]string)
 			for _, ID := range IDs {
 				id := string(fastx.ParseHeadID(idRe, []byte(ID)))
+				if ignoreCase {
+					id = strings.ToLower(id)
+				}
 				if _, ok := idsMap[id]; !ok {
 					idsMap[id] = []string{}
 				}
@@ -846,14 +906,14 @@ Examples:
 					checkError(fmt.Errorf("sorry, two-pass mode does not support FASTQ format"))
 				}
 			}
-			fileExt = suffixFA
+			fileExt = suffixFA + extension
 
 			if !quiet {
 				log.Infof("read sequence IDs and sequence region from FASTA file ...")
 			}
 			region2name := make(map[string][]string)
 
-			fastxReader, err = fastx.NewReader(alphabet2, newFile, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet2, newFile, idRegexp)
 			checkError(err)
 			var name string
 			var subseq string
@@ -880,15 +940,32 @@ Examples:
 				}
 				region2name[subseq] = append(region2name[subseq], name)
 			}
+			fastxReader.Close()
+
+			fileFai := newFile + ".seqkit.fai"
+
+			if FileExists(fileFai) && updateFaidx {
+				checkError(os.RemoveAll(fileFai))
+				if !quiet {
+					log.Infof("delete the old FASTA index file: %s", fileFai)
+				}
+			}
 
 			if !quiet {
-				log.Infof("create and read FASTA index ...")
+				log.Infof("create or read FASTA index ...")
 			}
-			faidx := getFaidx(newFile, `^(.+)$`)
+			faidx := getFaidx(newFile, `^(.+)$`, quiet)
 			defer func() {
 				checkError(faidx.Close())
 			}()
 
+			if len(faidx.Index) == 0 {
+				log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+				return
+			} else if !quiet {
+				log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
+			}
+
 			var wg sync.WaitGroup
 			tokens := make(chan int, config.Threads)
 
@@ -953,9 +1030,11 @@ func init() {
 	splitCmd.Flags().IntP("by-size", "s", 0, "split sequences into multi parts with N sequences")
 	splitCmd.Flags().IntP("by-part", "p", 0, "split sequences into N parts")
 	splitCmd.Flags().BoolP("by-id", "i", false, "split squences according to sequence ID")
+	splitCmd.Flags().BoolP("ignore-case", "I", false, "ignore case when using -i/--by-id")
 	splitCmd.Flags().StringP("by-region", "r", "", "split squences according to subsequence of given region. "+
 		`e.g 1:12 for first 12 bases, -12:-1 for last 12 bases. type "seqkit split -h" for more examples`)
 	splitCmd.Flags().BoolP("two-pass", "2", false, "two-pass mode read files twice to lower memory usage. (only for FASTA format)")
+	splitCmd.Flags().BoolP("update-faidx", "U", false, "update the fasta index file if it exists. Use this if you are not sure whether the fasta file changed")
 	splitCmd.Flags().BoolP("dry-run", "d", false, "dry run, just print message and no files will be created.")
 	splitCmd.Flags().BoolP("keep-temp", "k", false, "keep temporary FASTA and .fai file when using 2-pass mode")
 	splitCmd.Flags().StringP("out-dir", "O", "", "output directory (default value is $infile.split)")
