@@ -39,6 +39,8 @@ import (
 
 // split2Cmd represents the split command
 var split2Cmd = &cobra.Command{
+	GroupID: "set",
+
 	Use:   "split2",
 	Short: "split sequences into files by size/parts (FASTA, PE/SE FASTQ)",
 	Long: `split sequences into files by part size or number of parts
@@ -58,6 +60,16 @@ The extension of output files:
      gzipped files for plain text input:
          seqkit split2 -p 2 -O test tests/hairpin.fa -e .gz
 
+
+If you want to cut a sequence into multiple segments.
+  1. For cutting into even chunks, please use 'kmcp utils split-genomes'
+     (https://bioinf.shenwei.me/kmcp/usage/#split-genomes).
+     E.g., cutting into 4 segments of equal size, with no overlap between adjacent segments:
+        kmcp utils split-genomes -m 1 -k 1 --split-number 4 --split-overlap 0 input.fasta -O out
+  2. For cutting into multiple chunks of fixed size, please using 'seqkit sliding'.
+     E.g., cutting into segments of 40 bp and keeping the last segment which can be shorter than 40 bp.
+        seqkit sliding -g -s 40 -W 40 input.fasta -o out.fasta
+
 `,
 	Run: func(cmd *cobra.Command, args []string) {
 		if len(args) > 1 {
@@ -219,7 +231,6 @@ The extension of output files:
 
 				renameFileExt := true
 				var record *fastx.Record
-				var fastxReader *fastx.Reader
 				var err error
 
 				var outfhs []*xopen.Writer
@@ -242,7 +253,7 @@ The extension of output files:
 					checkError(fmt.Errorf(`one of flags should be given: -s/-p/-l. type "seqkit split2 -h" for help`))
 				}
 
-				fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+				fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 				checkError(err)
 				i := 0 // nth part
 				j := 0
@@ -399,8 +410,8 @@ The extension of output files:
 							i = 0
 						}
 					}
-
 				}
+				fastxReader.Close()
 
 				if byParts {
 					for i, outfh := range outfhs {
@@ -415,7 +426,9 @@ The extension of output files:
 						}
 					}
 				} else {
-					outfhPre.Close()
+					if outfhPre != nil {
+						outfhPre.Close()
+					}
 
 					if j == 0 {
 						os.Remove(outfilePre)
@@ -437,7 +450,7 @@ func init() {
 	split2Cmd.Flags().StringP("read1", "1", "", "(gzipped) read1 file")
 	split2Cmd.Flags().StringP("read2", "2", "", "(gzipped) read2 file")
 	split2Cmd.Flags().IntP("by-size", "s", 0, "split sequences into multi parts with N sequences")
-	split2Cmd.Flags().IntP("by-part", "p", 0, "split sequences into N parts")
+	split2Cmd.Flags().IntP("by-part", "p", 0, "split sequences into N parts with the round robin distribution")
 	split2Cmd.Flags().StringP("by-length", "l", "", "split sequences into chunks of >=N bases, supports K/M/G suffix")
 	split2Cmd.Flags().StringP("out-dir", "O", "", "output directory (default value is $infile.split)")
 	split2Cmd.Flags().BoolP("force", "f", false, "overwrite output directory")
