@@ -23,6 +23,7 @@ package cmd
 import (
 	"fmt"
 	"io"
+	"os"
 	"regexp"
 	"runtime"
 	"strconv"
@@ -39,17 +40,23 @@ import (
 
 // subseqCmd represents the subseq command
 var subseqCmd = &cobra.Command{
+	GroupID: "basic",
+
 	Use:   "subseq",
 	Short: "get subsequences by region/gtf/bed, including flanking sequences",
 	Long: fmt.Sprintf(`get subsequences by region/gtf/bed, including flanking sequences.
 
-Attentions:
-  1. Use "seqkit grep" for extract subsets of sequences.
+Attention:
+  1. When extracting with BED/GTF from plain text FASTA files, the order of output sequences
+     are random. To keep the order, just compress the FASTA file (input.fasta) and use the
+     compressed one (input.fasta.gz) as the input.
+  2. Use "seqkit grep" for extracting subsets of sequences.
      "seqtk subseq seqs.fasta id.txt" equals to
      "seqkit grep -f id.txt seqs.fasta"
 
 Recommendation:
-  1. use plain FASTA file, so seqkit could utilize FASTA index.
+  1. Use plain FASTA file, so seqkit could utilize FASTA index.
+  2. The flag -U/--update-faidx is recommended to ensure the .fai file matches the FASTA file.
 
 The definition of region is 1-based and with some custom design.
 
@@ -79,9 +86,10 @@ Examples:
 		chrs = chrs2
 		chrsMap := make(map[string]struct{}, len(chrs))
 		for _, chr := range chrs {
-			chrsMap[strings.ToLower(chr)] = struct{}{}
+			chrsMap[chr] = struct{}{}
 		}
 		region := getFlagString(cmd, "region")
+		appendRegionCoord := getFlagBool(cmd, "region-coord")
 
 		gtfFile := getFlagString(cmd, "gtf")
 		bedFile := getFlagString(cmd, "bed")
@@ -89,7 +97,7 @@ Examples:
 		choosedFeatures := getFlagStringSlice(cmd, "feature")
 		choosedFeatures2 := make([]string, len(choosedFeatures))
 		for i, f := range choosedFeatures {
-			choosedFeatures2[i] = strings.ToLower(f)
+			choosedFeatures2[i] = f
 		}
 		choosedFeatures = choosedFeatures2
 
@@ -112,6 +120,8 @@ Examples:
 			}
 		}
 
+		updateFaidx := getFlagBool(cmd, "update-faidx")
+
 		outfh, err := xopen.Wopen(outFile)
 		checkError(err)
 		defer outfh.Close()
@@ -158,11 +168,11 @@ Examples:
 
 			var chr, feat string
 			for _, feature := range features {
-				chr = strings.ToLower(feature.SeqName)
+				chr = feature.SeqName
 				if _, ok := gtfFeaturesMap[chr]; !ok {
 					gtfFeaturesMap[chr] = make(map[string][]gtf.Feature)
 				}
-				feat = strings.ToLower(feature.Feature)
+				feat = feature.Feature
 				if _, ok := gtfFeaturesMap[chr][feat]; !ok {
 					gtfFeaturesMap[chr][feat] = []gtf.Feature{}
 				}
@@ -191,7 +201,7 @@ Examples:
 
 			var chr string
 			for _, feature := range features {
-				chr = strings.ToLower(feature.Chr)
+				chr = feature.Chr
 				if _, ok := bedFeatureMap[chr]; !ok {
 					bedFeatureMap[chr] = []BedFeature{}
 				}
@@ -214,13 +224,33 @@ Examples:
 				id2name := make(map[string][]byte)
 
 				if !isFastq { // ok, it's fasta!
+					fileFai := file + ".seqkit.fai"
+
+					if FileExists(fileFai) && updateFaidx {
+						if !quiet {
+							checkError(os.RemoveAll(fileFai))
+							log.Infof("delete the old FASTA index file: %s", fileFai)
+						}
+					}
+
+					if !quiet {
+						log.Infof("create or read FASTA index ...")
+					}
+
 					// faidx, err := fai.New(file)
 					// checkError(err)
-					faidx := getFaidx(file, "^(.+)$")
+					faidx := getFaidx(file, idRegexp, quiet)
+
+					if len(faidx.Index) == 0 {
+						log.Warningf("  0 records loaded from %s, please check if it matches the fasta file, or switch on the flag -U/--update-faidx", fileFai)
+						return
+					} else if !quiet {
+						log.Infof("  %d records loaded from %s", len(faidx.Index), fileFai)
+					}
 
 					var id string
 					for head := range faidx.Index {
-						id = strings.ToLower(string(fastx.ParseHeadID(idRe, []byte(head))))
+						id = string(fastx.ParseHeadID(idRe, []byte(head)))
 						id2name[id] = []byte(head)
 					}
 
@@ -228,7 +258,7 @@ Examples:
 					if region != "" {
 						if len(chrs) > 0 {
 							for _, chr := range chrs {
-								chr2 = string(id2name[strings.ToLower(chr)])
+								chr2 = string(id2name[chr])
 								r, ok := faidx.Index[chr2]
 								if !ok {
 									log.Warningf(`sequence (%s) not found in file: %s`, chr2, file)
@@ -250,7 +280,7 @@ Examples:
 					} else if gtfFile != "" {
 						for chr := range gtfFeaturesMap {
 							if len(chrs) > 0 { // selected chrs
-								if _, ok := chrsMap[strings.ToLower(chr)]; !ok {
+								if _, ok := chrsMap[chr]; !ok {
 									continue
 								}
 							}
@@ -276,7 +306,7 @@ Examples:
 					} else if bedFile != "" {
 						for chr := range bedFeatureMap {
 							if len(chrs) > 0 { // selected chrs
-								if _, ok := chrsMap[strings.ToLower(chr)]; !ok {
+								if _, ok := chrsMap[chr]; !ok {
 									continue
 								}
 							}
@@ -303,14 +333,19 @@ Examples:
 
 					checkError(faidx.Close())
 				}
+
 			}
 
 			var record *fastx.Record
 			var fastxReader *fastx.Reader
 			// Parse all sequences
-			fastxReader, err = fastx.NewReader(alphabet, file, idRegexp)
+			fastxReader, err := fastx.NewReader(alphabet, file, idRegexp)
 			checkError(err)
 
+			var seqname string
+			var ok bool
+			noChrs := len(chrs) == 0
+
 			for {
 				record, err = fastxReader.Read()
 				if err != nil {
@@ -326,11 +361,16 @@ Examples:
 				}
 
 				if region != "" {
-					subseqByRegion(outfh, record, config.LineWidth, start, end)
-
+					if noChrs {
+						subseqByRegion(outfh, record, config.LineWidth, start, end, appendRegionCoord)
+					} else {
+						if _, ok = chrsMap[string(record.ID)]; ok {
+							subseqByRegion(outfh, record, config.LineWidth, start, end, appendRegionCoord)
+						}
+					}
 				} else if gtfFile != "" {
-					seqname := strings.ToLower(string(record.ID))
-					if _, ok := gtfFeaturesMap[seqname]; !ok {
+					seqname = string(record.ID)
+					if _, ok = gtfFeaturesMap[seqname]; !ok {
 						continue
 					}
 
@@ -339,8 +379,8 @@ Examples:
 						onlyFlank, upStream, downStream, gtfTag)
 
 				} else if bedFile != "" {
-					seqname := strings.ToLower(string(record.ID))
-					if _, ok := bedFeatureMap[seqname]; !ok {
+					seqname = string(record.ID)
+					if _, ok = bedFeatureMap[seqname]; !ok {
 						continue
 					}
 
@@ -349,6 +389,7 @@ Examples:
 						onlyFlank, upStream, downStream)
 				}
 			}
+			fastxReader.Close()
 
 			config.LineWidth = lineWidth
 		}
@@ -357,8 +398,11 @@ Examples:
 
 type type2gtfFeatures map[string][]gtf.Feature
 
-func subseqByRegion(outfh *xopen.Writer, record *fastx.Record, lineWidth int, start, end int) {
+func subseqByRegion(outfh *xopen.Writer, record *fastx.Record, lineWidth int, start, end int, appendRegionCoord bool) {
 	record.Seq = record.Seq.SubSeq(start, end)
+	if appendRegionCoord {
+		record.Name = []byte(fmt.Sprintf("%s:%d-%d %s", record.ID, start, end, record.Desc))
+	}
 	record.FormatToWriter(outfh, lineWidth)
 }
 
@@ -366,7 +410,7 @@ func subseqByGTFFile(outfh *xopen.Writer, record *fastx.Record, lineWidth int,
 	gtfFeaturesMap map[string]type2gtfFeatures, choosedFeatures []string,
 	onlyFlank bool, upStream int, downStream int, gtfTag string) {
 
-	seqname := strings.ToLower(string(record.ID))
+	seqname := string(record.ID)
 
 	var strand, tag, outname, flankInfo string
 	var s, e int
@@ -379,7 +423,7 @@ func subseqByGTFFile(outfh *xopen.Writer, record *fastx.Record, lineWidth int,
 
 	for featureType := range gtfFeaturesMap[seqname] {
 		if len(choosedFeatures) > 0 {
-			if _, ok := featsMap[strings.ToLower(featureType)]; !ok {
+			if _, ok := featsMap[featureType]; !ok {
 				continue
 			}
 		}
@@ -475,7 +519,7 @@ func subseqByGTFFile(outfh *xopen.Writer, record *fastx.Record, lineWidth int,
 func subSeqByBEDFile(outfh *xopen.Writer, record *fastx.Record, lineWidth int,
 	bedFeatureMap map[string][]BedFeature,
 	onlyFlank bool, upStream, downStream int) {
-	seqname := strings.ToLower(string(record.ID))
+	seqname := string(record.ID)
 
 	var strand, geneID, outname, flankInfo string
 	var s, e int
@@ -572,6 +616,7 @@ func init() {
 	subseqCmd.Flags().StringP("region", "r", "", "by region. "+
 		"e.g 1:12 for first 12 bases, -12:-1 for last 12 bases,"+
 		` 13:-1 for cutting first 12 bases. type "seqkit subseq -h" for more examples`)
+	subseqCmd.Flags().BoolP("region-coord", "R", false, "append coordinates to sequence ID for -r/--region")
 
 	subseqCmd.Flags().StringP("gtf", "", "", "by GTF (version 2.2) file")
 	subseqCmd.Flags().StringSliceP("feature", "", []string{}, `select limited feature types (multiple value supported, case ignored, only works with GTF)`)
@@ -580,4 +625,6 @@ func init() {
 	subseqCmd.Flags().BoolP("only-flank", "f", false, "only return up/down stream sequence")
 	subseqCmd.Flags().StringP("bed", "", "", "by tab-delimited BED file")
 	subseqCmd.Flags().StringP("gtf-tag", "", "gene_id", `output this tag as sequence comment`)
+
+	subseqCmd.Flags().BoolP("update-faidx", "U", false, "update the fasta index file if it exists. Use this if you are not sure whether the fasta file changed")
 }
